package jnditut.intro;

import javax.naming.*;
import javax.naming.directory.*;
import java.util.Properties;
import java.io.*;

import jnditut.fruit.*;

/**
 * This class exercises several JNDI Tutorial naming examples
 */
public class HelloNaming {
    public static void main( String[] args) {
        try {
			String initialURL = null;
			if (args.length != 0) {
				initialURL = args[0];
			}
            // Defining the contexts
            Context ctx = new InitialContext(initProperties(initialURL));
            
            // Looking up a file
            Object obj = ctx.lookup("report.txt");
            File file = (File) obj;
            System.out.println("File found: " + file.getPath());

            // Looking at the contents of a context
            System.out.println("----- Names out of context (NameClassPair)");
            NamingEnumeration list = ctx.list("awt"); // list returns a NamingEnumeration
            while (list.hasMore()) {
                NameClassPair nc = (NameClassPair)list.next(); // next() returns a NameClassPair
                System.out.println(nc);
            }
            
            // Looking at the contents of a context  (2) (uses more expensive Bindings - adds context info)
            System.out.println("----- Names in context (Binding)");
            NamingEnumeration bindings = ctx.listBindings("awt");
            while (bindings.hasMore()) {
                Binding bd = (Binding)bindings.next(); // Binding is a subclass of NameClassPair
                System.out.println(bd.getName() + ": " + bd.getObject());
            }
            
            // Binding, rebinding and unbinding
            System.out.println("Creating new Fruit object: new Fruit(\"cashew\")");
            Fruit fruit = new Fruit("cashew");
            
            System.out.println("Unbinding foobar and favorite");
            ctx.unbind("foobar");   // Unbinding objects possibly bound in previous run
            ctx.unbind("favorite");
            
            System.out.println("Binding 'favorite' to fruit");
            ctx.bind("favorite", fruit);
            
            System.out.println("Looking up 'favorite' ...");
            Fruit fruitFound = (Fruit)ctx.lookup("favorite");
            System.out.println("Fruit found: " + fruitFound);
            
            // Renaming
            //System.out.println("Renaming 'report.txt' to 'old_report.txt'");
            //ctx.rename("report.txt", "old_report.txt");
            
            // Creating and destroying a context
            //System.out.println("Creating context 'new'");
            //Context result = ctx.createSubcontext("new");
            //System.out.println("Destroying context 'new'");
            //ctx.destroySubcontext("new");
            
            System.out.println("--- Listing of root context");
            NamingEnumeration list2 = ctx.list("/"); // list returns a NamingEnumeration
            while (list2.hasMore()) {
                NameClassPair nc = (NameClassPair)list2.next(); // next() returns a NameClassPair
                System.out.println(nc);
            }


        } catch (NamingException e) {
            System.out.println("HelloNaming: " + e);
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("HelloNaming - property file not read: " + e);
            e.printStackTrace();
        }
    }
    
    /**
     * This method sets up initial properties for naming system.
     * Provider URL is loaded from properties file (jndi.properties) but
     * may also be read from -D argument 
     */
    public static Properties initProperties(String url) throws IOException {
        Properties env = System.getProperties(); // gets any system properties
        ClassLoader loader = ClassLoader.getSystemClassLoader();
        InputStream propsFile = loader.getResourceAsStream("jndi.properties");
        if (propsFile != null) {
            env.load(propsFile);
        }
		if (url != null) {
			env.put(Context.PROVIDER_URL, url);
			if(url.startsWith("file")) {
				env.put(Context.INITIAL_CONTEXT_FACTORY, "com.sun.jndi.fscontext.RefFSContextFactory");
			}
		}
		return env;
    }
}
